# Tags Changes

## 0.6.19

- Changed the `TreeView` to no longer dispatch `doubleClick_` when the double
click is on the expander; this is for symmetry with the changes made regarding
single click in the previous version.

## 0.6.18

- Changed the default behavior of items in the `TreeView` to not expand on a
single click; only a single click on the expander will cause the item to expand.
To expand on a single click anywhere in the item you may override the 
`singleClick_` method in your concrete `TreeView` class:

```dart
void singleClick_() => toggleExpanded();
```

## 0.6.17

- Changed the default theme font settings.

## 0.6.16

- Changed the `text` color of the `ThemeColorsDark` per feedback.

## 0.6.15

- Added a `ThemeDark` class.

## 0.6.14

- Fixed another focus issue when an item is selected in the `TreeView`.

## 0.6.13

- Added `fontSizePx` to the `ThemeDisplay` class.

## 0.6.12

- Changed the `KeyBinding` class to a concrete class.

## 0.6.11

- Removed the `isFirstAttach` parameter from the `Tree.attached_` method.

## 0.6.10

- Added the `removeChildren` method to the `Tree` and `TreeView` classes.

## 0.6.9

- Fixed an issue with focus when an item is selected in the `TreeView`.

## 0.6.8

- Changed the default `fontSize` of the default `Theme` to `14px`.

## 0.6.7

- Removed the svg tags; these were not being used in practice and removing them
frees up some common names in the top-level namespace.

## 0.6.6

- Added the `Editor` abstract class and the `EditorView` abstract element.

## 0.6.5

- Added the `Tree` abstract class and the `TreeView` abstract element.

## 0.6.4

- Added the `Selections` class to manage top-level selections; currently only
supports single-select.

## 0.6.3

- Added the top-level `routedCommands` map to the `tags` library.

## 0.6.2

- Added the `Command`, `KeyBinding` and `RoutedCommand` classes to the `tags`
library.
- Added the `CommandBinding` custom element to the `tags.html` library.

## 0.6.1

- Changed the return type of `EditableLabel.beginEdit` from `void` -> `Future`.
- Replaced usages of the deprecated `enteredView` and `leftView` callbacks with
`attached` and `detached`.

## 0.6.0

- Renamed the `TAGS_DOM` constant to `HTML` and the `tags.dom` library to
`tags.html`.
- Added the `down` attribute on `ToggleButton`; this attribute is added to the
element when the toggleable `isDown` and removed when it `isUp`.

## 0.5.6

- Replaced calls to the deprecated `document.register` with calls to 
`document.registerElement`.

## 0.5.5

- Added a `Theme` class for global styling of `Themeable` elements.
- Restored the use of shadow DOM to `EditableLabel` but without 
`applyAuthorStyles`; version `0.5.4` broke other aspects of this element.  It's 
inner elements can be styled from the outside in Chrome 35 using selectors such 
as `editable-label::shadow input[type="text"]` or from the inside by changing
its `resources` and / or the global `theme`.

## 0.5.4

- Removed the use of shadow DOM from `EditableLabel`; this element was using
`applyAuthorStyles` which does not exist in the upcoming Chrome 35.  We may add
this encapsulation back in a future revision after the platform stabilizes.

## 0.5.3

- Added an optional named parameter `select` to the `beginEdit` method of 
`EditableLabel` for selecting the label's text which defaults to `true`.
- Updated API documentation generation to use the new `docgen` tool.

## 0.5.2

- Fixed a styling issue in the `EditableLabel`.

## 0.5.1

- Capitalized the `TAGS_DOM` constant.

## 0.5.0

- Decoupled the `DOM` from the `tags` library by adding a `tags.dom` library;
the library user is responsible for setting the `dom` variable prior to making
any other calls to the `tags` library.  For example:

```dart
import 'package:tags/tags.dart' as tags;
import 'package:tags/dom.dart';

main() {
  tags.dom = TAGS_DOM;
  ...
}
```

- Removed the `registerTagsCustomElements` function; element registration is 
now performed when setting a value to the `dom` top-level variable.
- Added the `toggle` method to `Toggleable`.

## 0.4.7

- Implement a better workaround for dartbug.com/15401 below the API surface.

## 0.4.6

- Added the `isEditing`, `cancelEdit`, and `completeEdit` members to the 
`EditableLabel` and replaced the `onEndEdit` stream with `onEditChange`.

## 0.4.5

- Added the `Resources` and `Themeable` abstract classes.

## 0.4.4

- Implement a necessary workaround for dartbug.com/15401.

## 0.4.3

- Added an optional `displayFilter` field to the `EditableLabel`; when this is
not null it affects the rendered text for all calls to the `text` setter.
- Added an `onEndEdit` stream to the `EditableLabel`; one event will be added
to this stream for every call to `beginEdit` to signal the end of editing.
- Added `<circle>`, `<clipPath>`, `<defs>`, `<ellipse>`, `<g>`, `<line>`, 
`<marker>`, `<path>`, `<polygon>`, `<polyline>`, `<rect>`, `<svg>`, `<symbol>` 
and `<use>` tags.

## 0.4.2

- Updated for the Dart 1.0 release.

## 0.4.1

- Updated to SDK 0.8.10_r29803.
- Added `EditableLabel` custom element with `<editable-label>` tag and 
`editableLabel()` Tag function.

## 0.4.0

- Added `<article>`, `<aside>`, `<audio>`, `<footer>`, `<header>`, `<iframe>` 
and `<section>` tags.
- Removed the deprecated `HasElement` and `BindModel` types and the 
`bindModel` function.

## 0.3.0

- Changed `ToggleButton` and `ToggleOpen` from subtypes of `HasElement` to
subtypes of `ButtonElement` and `HtmlElement` respectively.
- Added the `children` function for building the children of an already created
element.

## 0.2.1

- Restored the `text` function name from `appendText`; thanks to user feedback!

## 0.2.0

- Added `<code>` tag.
- Added the `createTag` function to simplify `Tag` creation for tag authors.
- Renamed the `text` function to `appendText` to align better with the 
underlying call to `Element.appendText`.

## 0.1.9

- Moved all of the libraries into `lib/src/` and added a `lib/tags.dart` library
to expose the entire public API as a single import for users.  As part of this
change a number of function and class names were made longer to help avoid
conflicts.  This change was made to make the package easier to use, as a lot of
user code was importing two or three different libraries from this package and
that seemed unnecessary.

## 0.1.8

- Updated to SDK 0.6.19_r26297.
- Updated the `html` tag builder to use the new `HtmlHtmlElement` (renamed from
`HtmlElement`) in `dart:html`.  You buy posters!

## 0.1.7

- Removed export of `dart:html` from the `tags.html` library.  While this was
convenient, it is emerging as an anti-pattern in Dart because it can cause a lot
of confusion and errors for users who already import `dart:html` in their code.

## 0.1.6

- The `shadow` function in the `html` library now returns a `ShadowRoot`
instead of just a `Node`; this allows you to set properties on it directly such
as:

```dart
shadow(() {
  // your shadow dom
})..applyAuthorStyles = true;
```

## 0.1.5

- Updated to SDK 0.6.9_r25388.
- Added `<del>` tag.
- Some improvement to `ToggleOpen`.

## 0.1.4

- Renamed `DropDown` to `ToggleOpen` to more accurately reflect what it does; it
_can_ open as a drop-down but it could be styled to open up instead for example.
- More work on `ToggleOpen` including dartdoc.
- Renamed the `down` and `up` getters on `ToggleButton` to `isDown` and `isUp`.

## 0.1.3

- Updated to SDK 0.5.20.2_r24160.
- Prefix all library names with `tags`; for example `tags.html`.

## 0.1.2

- Added the function `append` in the `html` library; this provides a means to
append existing elements in a tag builder expression.
- Expose the `bind` function in the `custom` library; this is a convenience
function for implementors of custom tags.

## 0.1.1

- Updated to SDK 0.5.13_r23552.
- Added `<nav>` tag.
- Added an optional `args` to each `Tag`.  In the `html` library these are 
ignored.  In the `custom` library we have defined a `Bind` function type which
we take as the `args` for our custom tags.  This gives us a mechanism to deliver 
the instance of `HasElement` back to the user as a workaround for the `xtag` 
issue noted in the prior version.
- Added an experimental `DropDown` custom tag; this is a work in progress.

## 0.1.0

- Updated to SDK 0.5.9_r22879.
- ToggleButton can no longer store itself in the `xtag` of its `element` as the
type constraint for `xtag` was tightened to `Element` in this SDK version.  We 
need to wait for the ability to extend `Element` and for the custom element work 
in `dart:html` to settle down before we can make any design changes here, so for
now there is no way to backtrack from the `element` to its ToggleButton.

## 0.0.9

- Updated to SDK 0.5.7_r22611.
- Avoid streaming ToggleButton `states` events unless there is a listener and 
the stream is not paused. This avoids potential memory leaks that might arise 
with buffered events.

## 0.0.8

- Updated to SDK 0.5.5_r22416.

## 0.0.7

- Updated to SDK 0.4.7_r21548.

## 0.0.6

- Export `dart:html` library from `html.dart` so that users have only 1 import.
- Added `custom.dart` library and a ToggleButton to the showcase example.  The 
`custom.dart` library aims to provide a canonical set of custom elements (web 
components).
- The `build` functon in the `builder.dart` library now takes a `Create`
function to build the tag's element.  This allows us to differentiate between a 
builder function called within the scope of a `Create` (constructor / 
initializer of an Element) and the scope of a `Build` (child appending logic).

## 0.0.5

- Removed `tags.dart` library; users should import `html.dart` directly.
- Added `builder.dart` library to expose tag builder functionality; This library 
may be used for user-defined custom tags.

## 0.0.4

- Added `<i>`, `<u>`, `<dl>`, `<dt>`, and `<dd>` tags.
- Added `Build` typedef and some documentation to the API.
- Added `documentation` to pubspec.

## 0.0.3

- Added `<b>` tag.
- Added `text(String)` builder function to perform `Element.appendText(String)`
from within `build()` closures.

## 0.0.2

- Added basic usage to the README.

## 0.0.1

- Initial release.
