part of tags;

final Map<Symbol, RoutedCommand> _routedCommands = {
  #copy     : new RoutedCommand('copy'),
  #cut      : new RoutedCommand('cut'),
  #delete   : new RoutedCommand('delete'),
  #paste    : new RoutedCommand('paste'),
  #redo     : new RoutedCommand('redo'),
  #rename   : new RoutedCommand('rename'),  
  #undo     : new RoutedCommand('undo'),
  #zoomIn   : new RoutedCommand('zoom-in'),
  #zoomOut  : new RoutedCommand('zoom-out')
};

final Map<Symbol, RoutedCommand> routedCommands = new Map.from(_routedCommands);

class RoutedCommand extends Command {  
  
  /// The name of this routed command.
  final String name;
  
  /// The target of this routed command, may be `null`.
  Command target;
  
  /// The key bindings for this routed command.
  final List<KeyBinding> keys;
    
  bool get canExecute => target != null && target.canExecute;
    
  /// Creates a new routed command with the given [name] and key bindings.
  RoutedCommand(this.name, [List<KeyBinding> keys]) 
  : super._()
  , this.keys = (keys == null) ? <KeyBinding>[] : keys;
  
  Future call() {
    if (!canExecute) {
      return new Future.value();
    }
    return target();
  }
  
  /// Matches the given [keyEvent] against this routed command's [keys] and
  /// returns `true` if there is any match.
  bool matchKeys(keyEvent) => keys.fold(false, 
      (match, kb) => match || kb.matchKeys(keyEvent));
  
  String toString() => '$name($keys)';
}
