part of tags.html;

abstract class EditorView<T> extends DivElement implements Editor<T> {
    
  T _selection;
  T get selection => _selection;
    set selection(T selection) {
      if (_selection != selection) {
        _selection = selection;
        select_(_selection);
        _onSelectionChange.add(selection);
      }
    }
  
  bool get hasSelection => _selection != null;
    
  final _onSelectionChange = new StreamController<T>.broadcast();
  Stream<T> get onSelectionChange => _onSelectionChange.stream;
    
  dynamic _context;
  get context => _context;
  
  bool _isAttached = false;
  bool get isAttached => _isAttached;
  
  bool get isShown => !hidden;
  
  EditorView.created() : super.created();
  
  void initialize(T selection, [context]) {
    _selection = selection;
    _context = context;
    initialize_(selection, context);    
    select_(selection);
    children(this, () {   
      append(new CommandBinding(routedCommands[#copy], copy));
      append(new CommandBinding(routedCommands[#cut], cut));
      append(new CommandBinding(routedCommands[#paste], paste));
      append(new CommandBinding(routedCommands[#redo], redo));
      append(new CommandBinding(routedCommands[#undo], undo));
      append(new CommandBinding(routedCommands[#zoomIn], zoomIn));
      append(new CommandBinding(routedCommands[#zoomOut], zoomOut));
    });
  }
  
  void attached() {
    super.attached();    
    _isAttached = true;
    attached_();
  }
  
  void detached() {
    super.detached();
    _isAttached = false;
    detached_();
  }
  
  void show({bool select: true}) {
    hidden = false;
    show_();
    if (select) {
      Selections.select(selection);
    }
  }
  
  void hide() {
    hidden = true;
    hide_();
  }
  
  void shutdown() {
    shutdown_();
  }
    
// -----------------------------------------------------------------------------
// Editor stubs; concrete EditorViews should mixin an implementation.
// -----------------------------------------------------------------------------
  
  final Command copy = Command.noop;
  final Command cut = Command.noop;
  final Command paste = Command.noop;
  final Command redo = Command.noop;
  final Command undo = Command.noop;
  final Command zoomIn = Command.noop;
  final Command zoomOut = Command.noop;
  
  void initialize_(T data, context) {}
  void shutdown_() {}
  void attached_() {}
  void detached_() {}
  void show_() {}
  void hide_() {}
  void select_(T selection) {}
}
